
class Bloc:
    """ Un bloc dans un circuit. """
    
    def __init__(self, nom, entree_1, entree_2, sortie):
        """ Construit un nouveau bloc.
        
            :param nom: le nom servant à identifier le bloc.
            :param entree_1: la première entrée du bloc.
            :param entree_2: la seconde entrée du bloc.
            :param sortie: la sortie du bloc.
        """
        
        self.nom = nom
        self.entree_1 = entree_1 
        self.entree_2 = entree_2 
        self.sortie = sortie
        
    def __lt__(self, autre):
        """ Permet la comparaison avec un autre bloc. 
            Surtout utile pour trier une liste de blocs.
        """
        
        if not isinstance(autre, Bloc):
            raise ValueError("Seul un bloc peut être comparé à un bloc.")
        return self.nom < autre.nom
        
    def __eq__(self, autre):
        if not isinstance(autre, Bloc):
            raise ValueError("Seul un bloc peut être comparé à un bloc.")
        return autre.nom == self.nom
        
    def __hash__(self):
        return hash(self.nom)
        
    def __repr__(self):
        return 'Bloc({})'.format(self.nom)  
        
        